@@ -303,17 +303,22 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
             nemitt_y = scale_with_transverse_norm_emitt[1]
 
         if nemitt_x is None:
-            nemitt_x = 1
+            gemitt_x = 1
+        else:
+            gemitt_x = (nemitt_x / particle_ref._xobject.beta0[0]
+                        / particle_ref._xobject.gamma0[0])
+
         if nemitt_y is None:
-            nemitt_y = 1
+            gemitt_y = 1
+        else:
+            gemitt_y = (nemitt_y / particle_ref._xobject.beta0[0]
+                        / particle_ref._xobject.gamma0[0])
 
-        gemitt_x = nemitt_x/particle_ref.beta0/particle_ref.gamma0
-        gemitt_y = nemitt_y/particle_ref.beta0/particle_ref.gamma0
 
-        x_norm_scaled = np.sqrt(gemitt_x) * x_norm
-        px_norm_scaled = np.sqrt(gemitt_x) * px_norm
-        y_norm_scaled = np.sqrt(gemitt_y) * y_norm
-        py_norm_scaled = np.sqrt(gemitt_y) * py_norm
+        x_norm_scaled = np.sqrt(gemitt_x) * np.array(x_norm)
+        px_norm_scaled = np.sqrt(gemitt_x) * np.array(px_norm)
+        y_norm_scaled = np.sqrt(gemitt_y) * np.array(y_norm)
+        py_norm_scaled = np.sqrt(gemitt_y) * np.array(py_norm)
 
         # Transform long. coordinates to normalized space
         XX_long = np.zeros(shape=(6, num_particles), dtype=np.float64)
